package com.twofours.surespot.network;

import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;

import com.google.android.gcm.GCMRegistrar;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.JsonHttpResponseHandler;
import com.loopj.android.http.RequestParams;
import com.loopj.android.http.ResponseHandlerInterface;
import com.loopj.android.http.SyncHttpClient;
import com.twofours.surespot.CookieResponseHandler;
import com.twofours.surespot.IdentityController;
import com.twofours.surespot.activities.LoginActivity;
import com.twofours.surespot.common.SurespotConfiguration;
import com.twofours.surespot.common.SurespotConstants;
import com.twofours.surespot.common.SurespotLog;
import com.twofours.surespot.common.Utils;
import com.twofours.surespot.common.WebClientDevWrapper;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import ch.boye.httpclientandroidlib.HttpEntity;
import ch.boye.httpclientandroidlib.HttpException;
import ch.boye.httpclientandroidlib.HttpResponse;
import ch.boye.httpclientandroidlib.HttpResponseInterceptor;
import ch.boye.httpclientandroidlib.HttpStatus;
import ch.boye.httpclientandroidlib.client.CookieStore;
import ch.boye.httpclientandroidlib.client.HttpClient;
import ch.boye.httpclientandroidlib.client.methods.HttpGet;
import ch.boye.httpclientandroidlib.client.methods.HttpPost;
import ch.boye.httpclientandroidlib.cookie.Cookie;
import ch.boye.httpclientandroidlib.entity.mime.MultipartEntity;
import ch.boye.httpclientandroidlib.entity.mime.content.InputStreamBody;
import ch.boye.httpclientandroidlib.impl.client.BasicCookieStore;
import ch.boye.httpclientandroidlib.impl.client.DefaultHttpClient;
import ch.boye.httpclientandroidlib.protocol.HttpContext;
import cz.msebera.android.httpclient.Header;

public class NetworkController {
	protected static final String TAG = "NetworkController";
	private static String mBaseUrl;

	private Context mContext;
	private AsyncHttpClient mClient;
	private CookieStore mCookieStore;
	private SyncHttpClient mSyncClient;

	public void get(String url, RequestParams params, AsyncHttpResponseHandler responseHandler) {
		mClient.get(mBaseUrl + url, params, responseHandler);
	}

	public void post(String url, RequestParams params, AsyncHttpResponseHandler responseHandler) {
		mClient.post(mBaseUrl + url, params, responseHandler);
	}

	public CookieStore getCookieStore() {
		return mCookieStore;
	}

	private boolean mUnauthorized;

	public boolean isUnauthorized() {
		return mUnauthorized;
	}

	public synchronized void setUnauthorized(boolean unauthorized) {

		mUnauthorized = unauthorized;
		if (unauthorized) {
			mCookieStore.clear();
		}
	}

	public NetworkController(Context context) {
		mContext = context;
		mBaseUrl = SurespotConfiguration.getBaseUrl();
		mCookieStore = new BasicCookieStore();
		Cookie cookie = IdentityController.getCookie();
		if (cookie != null) {
			mCookieStore.addCookie(cookie);
		}

		mClient = new AsyncHttpClient();
		mSyncClient = new SyncHttpClient() ;

		HttpResponseInterceptor httpResponseInterceptor = new HttpResponseInterceptor() {

			@Override
			public void process(HttpResponse response, HttpContext context) throws HttpException, IOException {

				if (response.getStatusLine().getStatusCode() == HttpStatus.SC_UNAUTHORIZED) {
					String origin = context.getAttribute("http.cookie-origin").toString();

					if (origin != null) {

						if (!isUnauthorized()) {

							if (!(origin.contains(mBaseUrl.substring(7)) && origin.contains("/login"))) {
								setUnauthorized(true);

								mClient.cancelRequests(mContext, true);
								mSyncClient.cancelRequests(mContext, true);

								SurespotLog.v(TAG, "Got 401, launching login intent.");
								Intent intent = new Intent(mContext, LoginActivity.class);
								intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
								mContext.startActivity(intent);

							}

						}
					}
				}
			}
		};

		if (mClient != null && mSyncClient != null) {

			mClient.setCookieStore((cz.msebera.android.httpclient.client.CookieStore) mCookieStore);
			mSyncClient.setCookieStore((cz.msebera.android.httpclient.client.CookieStore) mCookieStore);

			// handle 401s
			//mClient.getDefaultHttpClient().addResponseInterceptor(httpResponseInterceptor);
			//mSyncClient.getDefaultHttpClient().addResponseInterceptor(httpResponseInterceptor);
		}
	}

	public void addUser(final String username, String password, String publicKey, final CookieResponseHandler responseHandler) {
		Map<String, String> params = new HashMap<String, String>();
		params.put("username", username);
		params.put("password", password);
		params.put("publickey", publicKey);
		// get the gcm id
		final String gcmIdReceived = Utils.getSharedPrefsString(mContext, SurespotConstants.PrefNames.GCM_ID_RECEIVED);

		boolean gcmUpdatedTemp = false;
		// update the gcmid if it differs
		if (gcmIdReceived != null) {

			params.put("gcmId", gcmIdReceived);
			gcmUpdatedTemp = true;
		}

		// just be javascript already
		final boolean gcmUpdated = gcmUpdatedTemp;

		post("/users", new RequestParams(params), new AsyncHttpResponseHandler() {

			@Override
			public void onSuccess(int responseCode, Header[] headers, byte[] result) {
				ch.boye.httpclientandroidlib.cookie.Cookie cookie = extractConnectCookie(mCookieStore);

				if (cookie == null) {
					SurespotLog.w(TAG, "did not get cookie from signup");
					responseHandler.onFailure(responseCode,headers,"Did not get cookie.".getBytes(),new Exception("Did not get cookie.") );
				}
				else {
					setUnauthorized(false);
					// update shared prefs
					if (gcmUpdated) {
						Utils.putSharedPrefsString(mContext, SurespotConstants.PrefNames.GCM_ID_SENT, gcmIdReceived);
					}

					responseHandler.onSuccess(responseCode,result.toString(),cookie);
				}

			}

			@Override
			public void onFailure(int code, Header[] headers, byte[] content, Throwable arg0) {
				responseHandler.onFailure(code, headers,content,arg0);
			}

			@Override
			public void onFinish() {
				responseHandler.onFinish();
			}

		});

	}

	private static Cookie extractConnectCookie(CookieStore cookieStore) {
		for (Cookie c : cookieStore.getCookies()) {
			// System.out.println("Cookie name: " + c.getName() + " value: " +
			// c.getValue());
			if (c.getName().equals("connect.sid")) {
				return c;
			}
		}
		return null;

	}

	public void login(String username, String password, final CookieResponseHandler responseHandler) {
		Map<String, String> params = new HashMap<String, String>();
		params.put("username", username);
		params.put("password", password);

		// get the gcm id
		final String gcmIdReceived = Utils.getSharedPrefsString(mContext, SurespotConstants.PrefNames.GCM_ID_RECEIVED);
		String gcmIdSent = Utils.getSharedPrefsString(mContext, SurespotConstants.PrefNames.GCM_ID_SENT);

		boolean gcmUpdatedTemp = false;
		// update the gcmid if it differs
		if (gcmIdReceived != null && !gcmIdReceived.equals(gcmIdSent)) {

			params.put("gcmId", gcmIdReceived);
			gcmUpdatedTemp = true;
		}

		// just be javascript already
		final boolean gcmUpdated = gcmUpdatedTemp;

		post("/login", new RequestParams(params), new AsyncHttpResponseHandler() {

			@Override
			public void onSuccess(int responseCode, Header[] headers, byte[] result) {
				Cookie cookie = extractConnectCookie(mCookieStore);
				if (cookie == null) {
					SurespotLog.w(TAG, "Did not get cookie from login.");
					responseHandler.onFailure(responseCode,headers,null,new Exception("Did not get cookie."));
				}
				else {
					// update shared prefs
					if (gcmUpdated) {
						Utils.putSharedPrefsString(mContext, SurespotConstants.PrefNames.GCM_ID_SENT, gcmIdReceived);
					}

					responseHandler.onSuccess(responseCode, result.toString(), cookie);
				}

			}

			@Override
			public void onFailure(int code, Header[] headers, byte[] content, Throwable arg0) {
				responseHandler.onFailure(code,headers,content,arg0);
			}

			@Override
			public void onFinish() {
				responseHandler.onFinish();
			}
		});

	}

	public void getFriends(AsyncHttpResponseHandler responseHandler) {
		get("/friends", null, responseHandler);
	}

	// if we have an id get the messages since the id, otherwise get the last x
	public void getMessages(String room, String id, AsyncHttpResponseHandler responseHandler) {

		if (id == null) {
			get("/messages/" + room, null, responseHandler);
		}
		else {
			get("/messages/" + room + "/after/" + id, null, responseHandler);
		}
	}

	// if we have an id get the messages since the id, otherwise get the last x
	public void getEarlierMessages(String room, String id, AsyncHttpResponseHandler responseHandler) {
		get("/messages/" + room + "/before/" + id, null, responseHandler);
	}

	public void getLastMessageIds(JsonHttpResponseHandler responseHandler) {
		get("/conversations/ids", null, responseHandler);
	}

	public void getPublicKey(String username, AsyncHttpResponseHandler responseHandler) {
		get("/publickey/" + username, null, responseHandler);

	}

	public String getPublicKeySync(String username) {
		return mSyncClient.get(mBaseUrl + "/publickey/" + username, new ResponseHandlerInterface() {
			@Override
			public void sendResponseMessage(cz.msebera.android.httpclient.HttpResponse response) throws IOException {

			}

			@Override
			public void sendStartMessage() {

			}

			@Override
			public void sendFinishMessage() {

			}

			@Override
			public void sendProgressMessage(long bytesWritten, long bytesTotal) {

			}

			@Override
			public void sendCancelMessage() {

			}

			@Override
			public void sendSuccessMessage(int statusCode, Header[] headers, byte[] responseBody) {

			}

			@Override
			public void sendFailureMessage(int statusCode, Header[] headers, byte[] responseBody, Throwable error) {

			}

			@Override
			public void sendRetryMessage(int retryNo) {

			}

			@Override
			public URI getRequestURI() {
				return null;
			}

			@Override
			public void setRequestURI(URI requestURI) {

			}

			@Override
			public Header[] getRequestHeaders() {
				return new Header[0];
			}

			@Override
			public void setRequestHeaders(Header[] requestHeaders) {

			}

			@Override
			public boolean getUseSynchronousMode() {
				return false;
			}

			@Override
			public void setUseSynchronousMode(boolean useSynchronousMode) {

			}

			@Override
			public boolean getUsePoolThread() {
				return false;
			}

			@Override
			public void setUsePoolThread(boolean usePoolThread) {

			}

			@Override
			public void onPreProcessResponse(ResponseHandlerInterface instance, cz.msebera.android.httpclient.HttpResponse response) {

			}

			@Override
			public void onPostProcessResponse(ResponseHandlerInterface instance, cz.msebera.android.httpclient.HttpResponse response) {

			}

			@Override
			public Object getTag() {
				return null;
			}

			@Override
			public void setTag(Object TAG) {

			}
		}).toString();
	}

	public void invite(String friendname, AsyncHttpResponseHandler responseHandler) {

		post("/invite/" + friendname, null, responseHandler);

	}

	public void respondToInvite(String friendname, String action, AsyncHttpResponseHandler responseHandler) {
		post("/invites/" + friendname + "/" + action, null, responseHandler);
	}

	public void registerGcmId(final AsyncHttpResponseHandler responseHandler) {
		// make sure the gcm is set
		// use case:
		// user signs-up without google account (unlikely)
		// user creates google account
		// user opens app again, we have session so neither login or add user is called (which wolud set the gcm)
		// so we need to upload the gcm here if we haven't already
		// get the gcm id

		final String gcmIdReceived = Utils.getSharedPrefsString(mContext, SurespotConstants.PrefNames.GCM_ID_RECEIVED);
		String gcmIdSent = Utils.getSharedPrefsString(mContext, SurespotConstants.PrefNames.GCM_ID_SENT);

		Map<String, String> params = new HashMap<String, String>();

		boolean gcmUpdatedTemp = false;
		// update the gcmid if it differs
		if (gcmIdReceived != null && !gcmIdReceived.equals(gcmIdSent)) {

			params.put("gcmId", gcmIdReceived);
			gcmUpdatedTemp = true;
		}
		else {
			SurespotLog.v(TAG, "GCM does not need updating on server.");
			return;
		}

		// just be javascript already
		final boolean gcmUpdated = gcmUpdatedTemp;

		post("/registergcm", new RequestParams(params), new AsyncHttpResponseHandler() {

			@Override
			public void onSuccess(int responseCode, Header[] headers, byte[] result) {

				// update shared prefs
				if (gcmUpdated) {
					Utils.putSharedPrefsString(mContext, SurespotConstants.PrefNames.GCM_ID_SENT, gcmIdReceived);
				}

				responseHandler.onSuccess(responseCode, headers, result);
			}

			@Override
			public void onFailure(int code, Header[] headers, byte[]arg1, Throwable arg0) {
				responseHandler.onFailure(code,headers,arg1, arg0);
			}

		});

	}

	public void validate(String username, String password, String publickey, AsyncHttpResponseHandler responseHandler) {
		RequestParams params = new RequestParams();

		params.put("username", username);
		params.put("password", password);
		if (publickey != null) {
			params.put("publickey", publickey);
		}

		// ideally would use a get here but putting body in a get request is frowned upon apparently:
		// http://stackoverflow.com/questions/978061/http-get-with-request-body
		// It's also not a good idea to put passwords in the url
		post("/validate", params, responseHandler);
	}

	public void userExists(String username, AsyncHttpResponseHandler responseHandler) {
		get("/users/" + username + "/exists", null, responseHandler);
	}

	/**
	 * Unregister this account/device pair within the server.
	 */
	public static void unregister(final Context context, final String regId) {
		SurespotLog.i(TAG, "unregistering device (regId = " + regId + ")");
		try {
			// this will puke on phone with no google account
			GCMRegistrar.setRegisteredOnServer(context, false);
		}
		finally {
		}
	}

	public void postFile(Context context, String user, String id, InputStream fileStream, String mimeType,
			AsyncHttpResponseHandler responseHandler) {

		RequestParams params = new RequestParams();
		params.put("image", fileStream, id, mimeType);

		post("/images/" + user, params, responseHandler);

	}

	public String postFileSync(Context context, String user, String id, InputStream fileStream, String mimeType) {

		RequestParams params = new RequestParams();
		params.put("image", fileStream, id, mimeType);

		return mSyncClient.post(mBaseUrl + "/images/" + user, params, new ResponseHandlerInterface() {
			@Override
			public void sendResponseMessage(cz.msebera.android.httpclient.HttpResponse response) throws IOException {

			}

			@Override
			public void sendStartMessage() {

			}

			@Override
			public void sendFinishMessage() {

			}

			@Override
			public void sendProgressMessage(long bytesWritten, long bytesTotal) {

			}

			@Override
			public void sendCancelMessage() {

			}

			@Override
			public void sendSuccessMessage(int statusCode, Header[] headers, byte[] responseBody) {

			}

			@Override
			public void sendFailureMessage(int statusCode, Header[] headers, byte[] responseBody, Throwable error) {

			}

			@Override
			public void sendRetryMessage(int retryNo) {

			}

			@Override
			public URI getRequestURI() {
				return null;
			}

			@Override
			public void setRequestURI(URI requestURI) {

			}

			@Override
			public Header[] getRequestHeaders() {
				return new Header[0];
			}

			@Override
			public void setRequestHeaders(Header[] requestHeaders) {

			}

			@Override
			public boolean getUseSynchronousMode() {
				return false;
			}

			@Override
			public void setUseSynchronousMode(boolean useSynchronousMode) {

			}

			@Override
			public boolean getUsePoolThread() {
				return false;
			}

			@Override
			public void setUsePoolThread(boolean usePoolThread) {

			}

			@Override
			public void onPreProcessResponse(ResponseHandlerInterface instance, cz.msebera.android.httpclient.HttpResponse response) {

			}

			@Override
			public void onPostProcessResponse(ResponseHandlerInterface instance, cz.msebera.android.httpclient.HttpResponse response) {

			}

			@Override
			public Object getTag() {
				return null;
			}

			@Override
			public void setTag(Object TAG) {

			}
		}).toString();

	}

	public void postFileStream(Context context, final String user, final String id, final InputStream fileInputStream,
			final String mimeType, final IAsyncCallback<Boolean> callback) {
		new AsyncTask<Void, Void, HttpResponse>() {

			@Override
			protected HttpResponse doInBackground(Void... params) {

				SurespotLog.v(TAG, "posting file stream"); DefaultHttpClient defaultHttpClient = new DefaultHttpClient();

				HttpClient httpclient = WebClientDevWrapper.wrapClient(defaultHttpClient);
				HttpPost httppost = new HttpPost(mBaseUrl + "/images/" + user);

				InputStreamBody isBody = new InputStreamBody(fileInputStream, mimeType, id);
				// StringBody comment = new StringBody("Filename: " + fileName);

				MultipartEntity reqEntity = new MultipartEntity();
				reqEntity.addPart("image", isBody);
				// reqEntity.addPart("comment", comment);
				httppost.setEntity(reqEntity);
				Cookie cookie = IdentityController.getCookie();
				httppost.addHeader("cookie", cookie.getName() + "=" + cookie.getValue());

				HttpResponse response = null;

				try {
					response = httpclient.execute(httppost);

				}
				catch (Exception e) {
					SurespotLog.w(TAG, "createPostFile", e);
				}
				return response;

			}

			protected void onPostExecute(HttpResponse response) {

				if (response.getStatusLine().getStatusCode() == 202) {

					callback.handleResponse(true);
				}
				else {
					callback.handleResponse(false);
				}
			};
		}.execute();

	}

	public InputStream getFileStream(Context context, final String url) {

		SurespotLog.v(TAG, "getting file stream");

		HttpClient httpclient = WebClientDevWrapper.wrapClient(new DefaultHttpClient());
		HttpGet httpGet = new HttpGet(mBaseUrl + url);

		Cookie cookie = IdentityController.getCookie();
		httpGet.addHeader("cookie", cookie.getName() + "=" + cookie.getValue());

		HttpResponse response;
		try {
			response = httpclient.execute(httpGet);
			HttpEntity resEntity = response.getEntity();
			if (response.getStatusLine().getStatusCode() == 200) {
				return resEntity.getContent();
			}

		}

		catch (Exception e) {
			SurespotLog.w(TAG, "getFileStream", e);

		}

		return null;

	}

	public void getFile(String relativeUrl, AsyncHttpResponseHandler responseHandler) {
		get(relativeUrl, null, responseHandler);
	}

	public String getFileSync(String relativeUrl) {
		return mSyncClient.get(mBaseUrl + relativeUrl, new ResponseHandlerInterface() {
			@Override
			public void sendResponseMessage(cz.msebera.android.httpclient.HttpResponse response) throws IOException {

			}

			@Override
			public void sendStartMessage() {

			}

			@Override
			public void sendFinishMessage() {

			}

			@Override
			public void sendProgressMessage(long bytesWritten, long bytesTotal) {

			}

			@Override
			public void sendCancelMessage() {

			}

			@Override
			public void sendSuccessMessage(int statusCode, Header[] headers, byte[] responseBody) {

			}

			@Override
			public void sendFailureMessage(int statusCode, Header[] headers, byte[] responseBody, Throwable error) {

			}

			@Override
			public void sendRetryMessage(int retryNo) {

			}

			@Override
			public URI getRequestURI() {
				return null;
			}

			@Override
			public void setRequestURI(URI requestURI) {

			}

			@Override
			public Header[] getRequestHeaders() {
				return new Header[0];
			}

			@Override
			public void setRequestHeaders(Header[] requestHeaders) {

			}

			@Override
			public boolean getUseSynchronousMode() {
				return false;
			}

			@Override
			public void setUseSynchronousMode(boolean useSynchronousMode) {

			}

			@Override
			public boolean getUsePoolThread() {
				return false;
			}

			@Override
			public void setUsePoolThread(boolean usePoolThread) {

			}

			@Override
			public void onPreProcessResponse(ResponseHandlerInterface instance, cz.msebera.android.httpclient.HttpResponse response) {

			}

			@Override
			public void onPostProcessResponse(ResponseHandlerInterface instance, cz.msebera.android.httpclient.HttpResponse response) {

			}

			@Override
			public Object getTag() {
				return null;
			}

			@Override
			public void setTag(Object TAG) {

			}
		}).toString();
	}

	public void logout(final AsyncHttpResponseHandler responseHandler) {
		post("/logout", null, new AsyncHttpResponseHandler() {
			@Override
			public void onSuccess(int statusCode, Header[]headers,byte[] content) {
				setUnauthorized(true);

				responseHandler.onSuccess(statusCode, headers,content);
			}

			@Override
			public void onFailure(int code, Header[] headers, byte[] content, Throwable error) {
				responseHandler.onFailure(code,headers,content,error);
			}

		});

	}

	public void clearCache() {
		//mClient.clearCache();
	}

}
